#include <iostream>
#include <fstream>
#include <assert.h>
#include "configParser.hh"
using namespace std;


bool isWhitespace(char c) 
{
  return (c == ' ' || c == '\n' || c == '\t' || c == '\r');
}

// Parses things like 'A BB "CC C" D' -- spaces separate words, except inside quotes.
vector<string>* parseQ(const string &s)
{
  unsigned int i = 0;
  unsigned int L = s.length();
  vector<string> *v = new vector<string>;

  while(1) {
    while (i < L && isWhitespace(s[i])) {
      ++i;
    }
      
    if (i == L)
      return v;
    
    string p = "";

    if (s[i] == '\"') {
      // Quoted -- read until "
      ++i; // skip the opening quote
      while (i < L && s[i] != '\"') {
        p += s[i];
        ++i;
      }
      
      if (i != L) {
        ++i; // skip the closing quote
      } else {
        cerr << "Warning: unterminated quoted string" << endl;
      }

    } else {
      // Unquoted -- read until whitespace
      while (i < L && !isWhitespace(s[i])) {
        p += s[i];
        ++i;
      }
    }
    
    v->push_back(p);
  }
}

// Returns NULL if the file or line is missing.
vector<string>* findConfigLine(const char *filename, const string &key)
{
  ifstream file;
  file.open(filename);
  
  if (!file.good()) {
    cerr << "Config file " << filename << " cannot be read." << endl;
    return NULL;
  }

  string line;
  
  unsigned int keyLen = key.length();

  while(getline(file, line)) {
    if (line.compare(0, keyLen, key) == 0) {
      if (line.length() == keyLen) {
        // Empty matching line
        return new vector<string>;
      }
      
      if (isWhitespace(line[keyLen])) {
        // Nonempty matching line
        return parseQ(line.substr(keyLen + 1, string::npos));
      }
      
      // Otherwise, it wasn't really a matching line.
    }
  }
    
  return NULL;
}

