#include <assert.h>
#include <iostream>
#include <sstream>
#include "reductionAlgs.hh"
using namespace std;

int smallestPowerOfTwoAtLeast(int n)
{
  int i = 1;
  while (1)
  {
    if (i >= n)
      return i;
    i *= 2;
    assert(i != 0);
  }
}


// Should implicitly serialize, back up, log, cache.
int testTargetWithTestcase(programInfo &p, testcase &t)
{
  t.writeToDefault();
  return p.test();
}


// Use a binary-search-like algorithm to reduce the testcase.
// Usually O(min(m * lg n, n)) in oracle calls, where m is the size of the minimal testcase and n is the size of the original testcase :)
// Worst-case O(n^2).
// Postcondition: 1-minimal :)
void reduceTestcase(programInfo &p, testcase &t, int desiredExitCode)
{
  int totalSize = t.parts.size();
  int chunkSize = smallestPowerOfTwoAtLeast(totalSize);
  if (chunkSize > 1)
    chunkSize = chunkSize / 2;
  
  while (1)
  {
    cout << "Chunk size: " << chunkSize << endl;
  
    int chunksRemoved = 0;
    int chunksSurviving = 0;
    
    for (int chunkStart = 0; chunkStart < totalSize; chunkStart += chunkSize) {
      int chunkEnd = min(totalSize, chunkStart + chunkSize);
      
      assert(chunkEnd > chunkStart);
      
      // Is this chunk of the testcase already completely suppressed?  If so, don't bother.
      bool testThisChunk = false;
      for (int pos = chunkStart; pos < chunkEnd; ++pos) {
        if (t.parts[pos]->suppressed == 0) {
          testThisChunk = true;
          break;
        }
      }

      //if (!testThisChunk)
      //  cout << "Skipping chunk " << chunkStart << ".." << (chunkEnd - 1) << endl;
      
      if (testThisChunk) {
        // Suppress this entire chunk.
        for (int pos = chunkStart; pos < chunkEnd; ++pos) {
          ++ t.parts[pos]->suppressed;
        }
        
        if (testTargetWithTestcase(p, t) == desiredExitCode) {
          cout << "Yay, reduced testcase by removing " << chunkStart << ".." << (chunkEnd - 1) << " :)" << endl;
          
          ++chunksRemoved;
        } else {
          cout << "Removing " << chunkStart << ".." << (chunkEnd - 1) << " made the problem go away.  Putting it back." << endl;

          ++chunksSurviving;

          // Revert (unsuppress this entire chunk).
          for (int pos = chunkStart; pos < chunkEnd; ++pos) {
            -- t.parts[pos]->suppressed;
          }
        }
      }

    }
    
    // Save it.
    ostringstream oss;
    oss << "didcs" << chunkSize << ".bak"; // this should be based on the testcase's filename somehow..
    string outfilename = oss.str();
    t.writeToFile(outfilename.c_str());
    
    cout << endl;
    cout << "Done with a round of chunk size " << chunkSize << "!" << endl;
    cout << chunksSurviving << " chunks survived; " << chunksRemoved << " were removed" << endl;
    cout << "I think that means the testcase now has " << (chunksSurviving * chunkSize) << " lines/whatevers left." << endl;
    cout << "Intermediate testcase saved as " << outfilename << endl;
    cout << endl;
        
    if (chunkSize == 1) {
      if (chunksRemoved == 0) {
        cout << "Done!" << endl;
        break; // We're completely done. The testcase is now 1-minimal.
      }
      // Otherwise, continue with the same chunk size (1) to see if we can eliminate any more lines on an extra pass.
    } else {
      // Always continue with the next smaller chunk size.
      chunkSize /= 2;
      assert(chunkSize != 0);
    }
    
    cout << "Continuing with chunk size " << chunkSize << endl;
  }
  
  // Print statistics: number of tests, original size, final size, time spent, a graph, whatever.


  t.writeToDefault();
}

